@@ -7,6 +7,7 @@ Components.utils.import("resource://calendar/modules/calUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Preferences.jsm");
+Components.utils.import("resource://calendar/modules/ltnInvitationUtils.jsm");
 
 function convertFromUnicode(aCharset, aSrc) {
     let unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
@@ -22,8 +23,8 @@ function calItipEmailTransport() {
     this.wrappedJSObject = this;
     this._initEmailTransport();
 }
-const calItipEmailTransportClassID = Components.ID("{d4d7b59e-c9e0-4a7a-b5e8-5958f85515f0}");
-const calItipEmailTransportInterfaces = [Components.interfaces.calIItipTransport];
+var calItipEmailTransportClassID = Components.ID("{d4d7b59e-c9e0-4a7a-b5e8-5958f85515f0}");
+var calItipEmailTransportInterfaces = [Components.interfaces.calIItipTransport];
 calItipEmailTransport.prototype = {
     classID: calItipEmailTransportClassID,
     QueryInterface: XPCOMUtils.generateQI(calItipEmailTransportInterfaces),
@@ -39,8 +40,8 @@ calItipEmailTransport.prototype = {
     mDefaultIdentity: null,
     mDefaultSmtpServer: null,
 
-    get scheme() "mailto",
-    get type() "email",
+    get scheme() { return "mailto"; },
+    get type() { return "email"; },
 
     mSenderAddress: null,
     get senderAddress() {
@@ -215,32 +216,51 @@ calItipEmailTransport.prototype = {
                 }
             }
             case (Components.interfaces.calIItipItem.AUTO): {
-                cal.LOG("sendXpcomMail: Found AUTO autoResponse type.");
-                let toList = "";
-                for each (let recipient in aToList) {
-                    // Strip leading "mailto:" if it exists.
-                    let rId = recipient.id.replace(/^mailto:/i, "");
-                    // Prevent trailing commas.
-                    if (toList.length > 0) {
-                        toList += ", ";
-                    }
-                    // Add this recipient id to the list.
-                    toList += rId;
+                // don't show log message in case of falling through
+                if (aItem.autoResponse == Components.interfaces.calIItipItem.AUTO) {
+                    cal.LOG("sendXpcomMail: Found AUTO autoResponse type.");
                 }
-                let mailFile = this._createTempImipFile(compatMode, toList, aSubject, aBody, aItem, identity);
+                let cbEmail = function (aVal, aInd, aArr) {
+                    let email = cal.getAttendeeEmail(aVal, true);
+                    if (!email.length) {
+                        cal.LOG("Invalid recipient for email transport: " + aVal.toString());
+                    }
+                    return email;
+                }
+                let toMap = aToList.map(cbEmail).filter(function (aVal, aInd, aArr) {return (aVal.length)});
+                if (toMap.length < aToList.length) {
+                    // at least one invalid recipient, so we skip sending for this message
+                    return false;
+                }
+                let toList = toMap.join(', ');
+                let composeUtils = Components.classes["@mozilla.org/messengercompose/computils;1"]
+                                             .createInstance(Components.interfaces.nsIMsgCompUtils);
+                let messageId = composeUtils.msgGenerateMessageId(identity);
+                let mailFile = this._createTempImipFile(compatMode, toList, aSubject, aBody, aItem, identity, messageId);
                 if (mailFile) {
                     // compose fields for message: from/to etc need to be specified both here and in the file
                     let composeFields = Components.classes["@mozilla.org/messengercompose/composefields;1"]
                                                   .createInstance(Components.interfaces.nsIMsgCompFields);
                     composeFields.characterSet = "UTF-8";
                     composeFields.to = toList;
-                    composeFields.from = identity.email;
+                    let mailfrom = (!identity.fullName.length) ? identity.email : identity.fullName + " <" + identity.email + ">";
+                    composeFields.from = (cal.validateRecipientList(mailfrom) == mailfrom)
+                                         ? mailfrom : identity.email;
                     composeFields.replyTo = identity.replyTo;
+                    composeFields.organization = identity.organization;
+                    composeFields.messageId = messageId;
+                    let validRecipients;
                     if (identity.doCc) {
-                        composeFields.cc = cal.validateRecipients(identity.doCcList);
+                        validRecipients = cal.validateRecipientList(identity.doCcList);
+                        if (validRecipients != "") {
+                            composeFields.cc = validRecipients;
+                        }
                     }
                     if (identity.doBcc) {
-                        composeFields.bcc = cal.validateRecipients(identity.doBccList);
+                        validRecipients = cal.validateRecipientList(identity.doBccList);
+                        if (validRecipients != "") {
+                            composeFields.bcc = validRecipients;
+                        }
                     }
 
                     // xxx todo: add send/progress UI, maybe recycle
@@ -279,21 +299,8 @@ calItipEmailTransport.prototype = {
         return false;
     },
 
-    _createTempImipFile: function cietCTIF(compatMode, aToList, aSubject, aBody, aItem, aIdentity) {
+    _createTempImipFile: function (compatMode, aToList, aSubject, aBody, aItem, aIdentity, aMessageId) {
         try {
-            function encodeUTF8(text) {
-                return convertFromUnicode("UTF-8", text).replace(/(\r\n)|\n/g, "\r\n");
-            }
-            function encodeMimeHeader(header) {
-                let fieldNameLen = (header.indexOf(": ") + 2);
-                return MailServices.mimeConverter
-                                   .encodeMimePartIIStr_UTF8(header,
-                                                             false,
-                                                             "UTF-8",
-                                                             fieldNameLen,
-                                                             Components.interfaces.nsIMimeConverter.MIME_ENCODED_WORD_SIZE);
-            }
-
             let itemList = aItem.getItemList({});
             let serializer = Components.classes["@mozilla.org/calendar/ics-serializer;1"]
                                        .createInstance(Components.interfaces.calIIcsSerializer);
@@ -302,24 +309,12 @@ calItipEmailTransport.prototype = {
             methodProp.value = aItem.responseMethod;
             serializer.addProperty(methodProp);
             let calText = serializer.serializeToString();
-            let utf8CalText = encodeUTF8(calText);
+            let utf8CalText = ltn.invitation.encodeUTF8(calText);
 
             // Home-grown mail composition; I'd love to use nsIMimeEmitter, but it's not clear to me whether
             // it can cope with nested attachments,
             // like multipart/alternative with enclosed text/calendar and text/plain.
-            let mailText = ("MIME-version: 1.0\r\n" +
-                            (aIdentity.replyTo
-                             ? "Return-path: " + aIdentity.replyTo + "\r\n" : "") +
-                            "From: " + aIdentity.email + "\r\n" +
-                            "To: " + aToList + "\r\n" +
-                            "Date: " + (new Date()).toUTCString() + "\r\n" +
-                            "Subject: " + encodeMimeHeader(aSubject.replace(/(\n|\r\n)/, "|")) + "\r\n");
-                            if (aIdentity.doCc) {
-                                mailText += ("Cc: " + cal.validRecipients(aIdentity.doCcList) + "\r\n");
-                            }
-                            if (aIdentity.doBcc) {
-                                mailText += ("Bcc: " + cal.validRecipients(aIdentity.doBccList) + "\r\n");
-                            }
+            let mailText = ltn.invitation.getHeaderSection(aMessageId, aIdentity, aToList, aSubject);
             switch (compatMode) {
                 case 1:
                     mailText += ("Content-class: urn:content-classes:calendarmessage\r\n" +
@@ -340,7 +335,7 @@ calItipEmailTransport.prototype = {
                                  "Content-type: text/plain; charset=UTF-8\r\n" +
                                  "Content-transfer-encoding: 8BIT\r\n" +
                                  "\r\n" +
-                                 encodeUTF8(aBody) +
+                                 ltn.invitation.encodeUTF8(aBody) +
                                  "\r\n\r\n\r\n" +
                                  "--Boundary_(ID_ryU4ZdJoASiZ+Jo21dCbwA)\r\n" +
                                  "Content-type: text/calendar; method=" + aItem.responseMethod + "; charset=UTF-8\r\n" +
